;;;; run.scm


(use awk test regex)

(define (count-lines in)
  (awk (read-line in) (line) ([nlines 0])
       ("^[ \t]*;" nlines)
       (else (add1 nlines))))

(define (max-linelength in)
  (awk (read-line in) (line) ([max-len 0])
       (#t (max max-len (string-length line)))))

(define (strip-blank in)
  (awk (read-line in) (line) ([lines '()])
       ("[^ \t]" (cons line lines))
       (else     lines)))

(define (read-passwd in)
  (let ([line (read-line in)])
    (if (eof-object? line)
	(values line #f)
	(values line (string-split-fields ":" line #:infix)))))

(define (sort-passwd in)
  (sort
   (awk (read-passwd in) (line fields) ([ans '()])
	(#t (cons (cons (car fields) line) ans)) ) 
   (lambda (x y) (string<? (car x) (car y)))))


(define passwd 
#<<EOF
root:x:0:0:root:/root:/bin/bash
daemon:x:1:1:daemon:/usr/sbin:/bin/sh
bin:x:2:2:bin:/bin:/bin/sh
sys:x:3:3:sys:/dev:/bin/sh
sync:x:4:65534:sync:/bin:/bin/sync
games:x:5:60:games:/usr/games:/bin/sh
man:x:6:12:man:/var/cache/man:/bin/sh
lp:x:7:7:lp:/var/spool/lpd:/bin/sh
mail:x:8:8:mail:/var/mail:/bin/sh
news:x:9:9:news:/var/spool/news:/bin/sh
EOF
)

(define sorted-passwd
  `(("bin" . "bin:x:2:2:bin:/bin:/bin/sh") 
    ("daemon" . "daemon:x:1:1:daemon:/usr/sbin:/bin/sh") 
    ("games" . "games:x:5:60:games:/usr/games:/bin/sh") 
    ("lp" . "lp:x:7:7:lp:/var/spool/lpd:/bin/sh") 
    ("mail" . "mail:x:8:8:mail:/var/mail:/bin/sh") 
    ("man" . "man:x:6:12:man:/var/cache/man:/bin/sh") 
    ("news" . "news:x:9:9:news:/var/spool/news:/bin/sh") 
    ("root" . "root:x:0:0:root:/root:/bin/bash") 
    ("sync" . "sync:x:4:65534:sync:/bin:/bin/sync") 
    ("sys" . "sys:x:3:3:sys:/dev:/bin/sh")))

(define input
 #<<EOF
;; This is a test

(define (test)
  (print "This is a test."))
EOF
)

(test-group "awk test"

  (test "count-lines" 3 (call-with-input-string input count-lines ))
  (test "max-linelength" 28 (call-with-input-string input max-linelength ))
  (test "strip-blank" `(";; This is a test" "(define (test)"  "  (print \"This is a test.\"))" )
	(reverse (call-with-input-string input strip-blank )))
  (test "sort-passwd" sorted-passwd (call-with-input-string passwd sort-passwd ))
)
